/**
 ******************************************************************************
 * @file    demo_msg_sub_push.c
 * @version V1.5.0
 * @date    2019/07/31
 * @brief   The file define PSDK message push command callback functions for linux operating system.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "demo_msg_sub_func.h"
#include "app_utils.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Exported variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
 * @brief PSDK data transmission bandwidth message push command callback function.
 * @param bandWidthMsg Data transmission bandwidth message structure.
 * @return None.
 */
static void bandWidthMsgCallback(const T_BandWidthMsg *bandWidthMsg)
{
    UNUSED(bandWidthMsg);

    LOG_TEST("Demo : Receive A BandWidth Msg");
    LOG_TEST(
            "maxUpComPortBandWidth: %dByte/s, maxDownComPortBandWidth: %dByte/s, maxDownNetPortBandWidth: %dKbps, maxOsdkUpChannBw: %dByte/s",
            bandWidthMsg->maxUpComPortBandWidth, bandWidthMsg->maxDownComPortBandWidth,
            bandWidthMsg->maxDownNetPortBandWidth, bandWidthMsg->maxOsdkUpChannBw);
    LOG_TEST("isUpComPortOverrun: %d, isDownComPortOverrun: %d, isDownNetPortOverrun: %d, isOsdkUpChannOverrun: %d",
             bandWidthMsg->isUpComPortOverrun, bandWidthMsg->isDownComPortOverrun, bandWidthMsg->isDownNetPortOverrun,
             bandWidthMsg->isOsdkUpChannOverrun);
    LOG_TEST(
            "upComPortBandWidth: %dByte/s, downComPortBandWidth: %dByte/s, downVideoStreamBandWidth: %dKbps, downDataStreamBandWidth: %dKbps, osdkUpChannBw: %dByte/s",
            bandWidthMsg->upComPortBandWidth, bandWidthMsg->downComPortBandWidth,
            bandWidthMsg->downVideoStreamBandWidth, bandWidthMsg->downDataStreamBandWidth, bandWidthMsg->osdkUpChannBw);
}

/**
 * @brief PSDK UAV attitude message push command callback function.
 * @param uavAttitudeMsg UAV attitude message structure.
 * @return None.
 */
static void uavAttitudeMsgCallback(const T_UavAttitudeMsg *uavAttitudeMsg)
{
    UNUSED(uavAttitudeMsg);

    LOG_TEST("Demo : Receive A Attitude Msg");
    LOG_TEST("Quaternion: %f %f %f %f", uavAttitudeMsg->q0, uavAttitudeMsg->q1, uavAttitudeMsg->q2, uavAttitudeMsg->q3);
}

/**
 * @brief PSDK battery information push command callback function.
 * @param batteryInfoMsg UAV battery information structure.
 * @param batteryInfoAck Battery information push command ack structure, specifies payload power-off ready state.
 * @return None.
 */
static void batteryInfoMsgCallback(const T_BatteryInfoMsg *batteryInfoMsg, T_BatteryInfoAck *batteryInfoAck)
{
    UNUSED(batteryInfoMsg);

    LOG_TEST("Demo : Receive A Battery Info Msg");
    LOG_TEST("capacityPercent: %d, powerOffNotice: %d", batteryInfoMsg->capacityPercent,
             batteryInfoMsg->powerOffNotice);

    /*
    * Users can add power off preparation work here, but if execution time of this part is too long,
    * please put it to anther task.
    */

    batteryInfoAck->powerOffReady = PSDK_PAYLOAD_POWER_OFF_READY;
}

/**
 * @brief PSDK GPS information push command callback function.
 * @param gpsInfoMsg GPS information structure.
 * @return None.
 */
static void gpsInfoMsgCallback(const T_GPSInfoMsg *gpsInfoMsg)
{
    UNUSED(gpsInfoMsg);

    LOG_TEST("Demo : Receive A Gps Info Msg");
    LOG_TEST("gpsState: %d", gpsInfoMsg->gpsState);
}

/**
 * @brief PSDK UAV state message push command callback function.
 * @param uavStateMsg UAV state message structure.
 * @return None.
 */
static void uavStateMsgCallback(const T_UavStateMsg *uavStateMsg)
{
    UNUSED(uavStateMsg);

    LOG_TEST("Demo : Receive A UAV State Msg");
    LOG_TEST("compassState: %d, motorState: %d, landState: %d, flightTime: %d*0.1s, uavType: %d",
             uavStateMsg->uavState.compassState, uavStateMsg->uavState.motorState, uavStateMsg->uavState.landState,
             uavStateMsg->flightTime, uavStateMsg->uavType);
}

/**
 * @brief PSDK mobile APP date and time message push command callback function.
 * @param appDateTimeMsg Mobile APP date and time message structure.
 * @return None.
 */
static void appDateTimeMsgCallback(const T_AppDateTimeMsg *appDateTimeMsg)
{
    UNUSED(appDateTimeMsg);

    LOG_TEST("Demo : Receive A App Date Time Msg");
    LOG_TEST("appTime: %d.%d.%d %d:%d:%d", appDateTimeMsg->year, appDateTimeMsg->month, appDateTimeMsg->day,
             appDateTimeMsg->hour, appDateTimeMsg->minute, appDateTimeMsg->second);
}

/**
 * @brief PSDK barometer height data message push command callback function.
 * @param baroHeightDataMsg Barometer height data message structure.
 * @return None.
 */
static void baroHeightDataMsgCallback(const T_baroHeightDataMsg *baroHeightDataMsg)
{
    UNUSED(baroHeightDataMsg);

    LOG_TEST("Demo : Receive A Baro Height Data Msg");
}

/**
 * @brief PSDK GPS raw data push command callback function.
 * @param gpsRawDataMsg GPS raw data structure.
 * @return None.
 */
static void gpsRawDataMsgCallback(const T_GpsRawDataMsg *gpsRawDataMsg)
{
    UNUSED(gpsRawDataMsg);

    LOG_TEST("Demo : Receive A Gps Raw Data Msg");
}

/**
 * @brief PSDK RTK raw data push command callback function.
 * @param rtkRawDataMsg RTK raw data structure.
 * @return None.
 */
static void rtkRawDataMsgCallback(const T_RtkRawDataMsg *rtkRawDataMsg)
{
    UNUSED(rtkRawDataMsg);

    LOG_TEST("Demo : Receive A Rtk Raw Data Msg");
}

/**
 * @brief PSDK UTC timestamp push command callback function.
 * @param utcTimestampMsg UTC timestamp data structure.
 * @return None.
 */
static void utcTimestampMsgCallback(const T_UtcTimestampMsg *utcTimestampMsg)
{
    UNUSED(utcTimestampMsg);

    LOG_TEST("Receive UtcTimestamp Msg");
    LOG_TEST("utcTime: %d.%d.%d %d:%d:%d.%06d", utcTimestampMsg->year, utcTimestampMsg->month, utcTimestampMsg->day,
             utcTimestampMsg->hour, utcTimestampMsg->minute, utcTimestampMsg->second, utcTimestampMsg->microsecond);
}

/**
 * @brief PSDK type of other payload mounted on UAV push command callback function.
 * @param otherPayloadTypeMsg other payload type message structure.
 * @return None.
 */
static void otherPayloadTypeMsgCallback(const T_OtherPayloadTypeMsg *otherPayloadTypeMsg)
{
    UNUSED(otherPayloadTypeMsg);

    LOG_TEST("Demo : Receive A Other Payload Type Msg");
    LOG_TEST("mainPayloadType: %d", otherPayloadTypeMsg->mainPayloadType);
}

/**
 * @brief PSDK focal length of other payload mounted on UAV push command callback function.
 * @param otherPayloadFocalLengthMsg other payload focal length message structure.
 * @return None.
 */
static void otherPayloadFocalLengthMsgCallback(const T_OtherPayloadFocalLengthMsg *otherPayloadFocalLengthMsg)
{
    UNUSED(otherPayloadFocalLengthMsg);

    LOG_TEST("Demo : Receive A Other Payload Focal Length Msg.");
    LOG_TEST("mainPayloadFocalLength: %d*0.1mm", otherPayloadFocalLengthMsg->mainPayloadFocalLength);
}

//@formatter:off
//PSDK message push command callback functions
T_PsdkPushDataMsgCallbackList g_msgCallbackList = {
        .bandWidthMsgCallback = bandWidthMsgCallback,
        .uavAttitudeMsgCallback= uavAttitudeMsgCallback,
        .batteryInfoMsgCallback = batteryInfoMsgCallback,
        .gpsInfoMsgCallback = gpsInfoMsgCallback,
        .uavStateMsgCallback = uavStateMsgCallback,
        .appDateTimeMsgCallback = appDateTimeMsgCallback,
        .baroHeightDataMsgCallback = baroHeightDataMsgCallback,
        .gpsRawDataMsgCallback = gpsRawDataMsgCallback,
        .rtkRawDataMsgCallback = rtkRawDataMsgCallback,
        .otherPayloadTypeMsgCallback = otherPayloadTypeMsgCallback,
        .otherPayloadFocalLengthMsgCallback = otherPayloadFocalLengthMsgCallback,
};
//@formatter:on

/* Exported functions --------------------------------------------------------*/

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
