﻿/**
 ******************************************************************************
 * @file    main.c
 * @version V1.5.0
 * @date    2019/07/31
 * @brief   An example program of PSDK for linux operating system.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "stdint.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <psdk_core.h>
#include <unistd.h>
#include <psdk_upper.h>
#include <psdk_msg_subcribe.h>
#include <psdk_app_func.h>
#include "demo_msg_sub_func.h"
#include <string.h>
#include "app_info.h"
#include "app_utils.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/

//use Manifold 2 UART 1
#define LINUX_UART_DEV "/dev/ttyTHS2"

/* Private variables ---------------------------------------------------------*/
//string used to test PSDK send
static char s_testSend[] = "test send data";
//UART device descriptor
static int s_psdkUartFd;
//PSDK protocol handle structure
static T_PsdkUpper s_psdkUpper;
//PSDK user information
static T_PsdkUserInfo s_testUserInfo;

//buffer used to store data received from mobile APP
static char s_receivePrint[1024];

#if SKYPORT_FW_FIX_VER_NEED
//fixed SKYPORT FW version
static T_PsdkUserFixSkyport s_fixSkyport = {0};
#endif

#if PSDK_APP_ALIAS_NEED
//PSDK application alias
static T_PsdkUserCustomInfo s_productAlias;
#endif

/* Exported variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
 * @brief Check user information string and fill in user information to structure.
 * @param userInfo Pointer to user information structure to be written.
 * @return None.
 */
static void FillInUserInfo(T_PsdkUserInfo *userInfo)
{
    memset(userInfo->APP_Name, 0, sizeof(userInfo->APP_Name));
    memset(userInfo->APP_Id, 0, sizeof(userInfo->APP_Id));
    memset(userInfo->APP_Key, 0, sizeof(userInfo->APP_Key));
    memset(userInfo->DeveloperAccount, 0, sizeof(userInfo->DeveloperAccount));

    if (strlen(USER_APP_NAME) <= sizeof(userInfo->APP_Name) &&
        strlen(USER_APP_ID) <= sizeof(userInfo->APP_Id) &&
        strlen(USER_APP_KEY) <= sizeof(userInfo->APP_Key) &&
        strlen(USER_DEVELOPER_ACCOUNT) <= sizeof(userInfo->DeveloperAccount)) {
        strcpy(userInfo->APP_Name, USER_APP_NAME);
        strcpy(userInfo->APP_Id, USER_APP_ID);
        strcpy(userInfo->APP_Key, USER_APP_KEY);
        strcpy(userInfo->DeveloperAccount, USER_DEVELOPER_ACCOUNT);
    } else {
        PSDK_LOG_ERROR("Length of user information string is beyond limit. Please check.\r\n");
    }

    if (!strcmp(userInfo->APP_Name, "your_app_name") ||
        !strcmp(userInfo->APP_Id, "your_app_id") ||
        !strcmp(userInfo->APP_Key, "your_app_key") ||
        !strcmp(userInfo->DeveloperAccount, "your_developer_account")) {
        PSDK_LOG_ERROR("Please fill in correct user information to app_infor.h file.\r\n");
    }
}

/**
 * @brief Send data to UART, PSDK loe level communication function.
 * @param pBuf Pointer to data to be sent.
 * @param bufLen Size of data to be sent.
 * @return PSDK function process state.
 */
static E_PsdkStat sendDataToUart(const uint8_t *pBuf, uint16_t bufLen)
{
    int32_t realLen;

    realLen = write(s_psdkUartFd, pBuf, bufLen);
    if (realLen == bufLen) {
        return PSDK_STAT_OK;
    } else {
        return PSDK_STAT_ERR;
    }
}

/* Exported functions --------------------------------------------------------*/

/**
 * @brief Process data received from mobile APP function.
 * @param pData Received data.
 * @param dataLen Received data length.
 * @return None.
 */
void ReceiveFromApp(const uint8_t *pData, uint16_t dataLen)
{
    //test receive from app
    memcpy(s_receivePrint, pData, dataLen);
    s_receivePrint[dataLen] = 0;
    LOG_TEST("Demo : Receive Data From App, Data is %s", s_receivePrint);
}

/**
 * @brief Receive and process PSDK data frame thread.
 * @param arg
 * @return
 */
void *psdkParse(void *arg)
{
    uint16_t realSendLen;
    uint8_t dataRec[128] = {0};
    int32_t recLen = 0;

    UNUSED(arg);

    while (1) {
        recLen = read(s_psdkUartFd, dataRec, sizeof(dataRec));
        if (recLen > 0) {
            PsdkUpper_ProcessReceiveData(&s_psdkUpper, dataRec, recLen);
        }
        usleep(10 * 1000);

        //test send to app
        PsdkAppFunc_TransferToApp(&s_psdkUpper, (const uint8_t *) s_testSend, (uint16_t) (strlen(s_testSend) + 1),
                                  &realSendLen);
        //test send to OSDK
        PsdkOsdkFunc_TransferToOsdk(&s_psdkUpper, (const uint8_t *) s_testSend, (uint16_t) (strlen(s_testSend) + 1),
                                    &realSendLen);
    }
}

/**
 * @brief UART initialize.
 * @param port Pointer to linux device.
 * @retval -1: Device initialize not successfully.
 *          not -1: Device descriptor.
 */
int UART_Init(const char *port)
{
    int fd;
    struct termios options;

    fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) {
        goto out;
    }

    if (tcgetattr(fd, &options) != 0) {
        close(fd);
        fd = -1;
        goto out;
    }

    cfsetispeed(&options, B115200);
    cfsetospeed(&options, B115200);

    options.c_cflag |= CLOCAL;
    options.c_cflag |= CREAD;
    options.c_cflag &= ~CRTSCTS;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= CS8;
    options.c_cflag &= ~PARENB;
    options.c_iflag &= ~INPCK;
    options.c_cflag &= ~CSTOPB;
    options.c_oflag &= ~OPOST;
    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
    options.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
    options.c_cc[VTIME] = 0;
    options.c_cc[VMIN] = 0;

    tcflush(fd, TCIFLUSH);

    if (tcsetattr(fd, TCSANOW, &options) != 0) {
        close(fd);
        fd = -1;
        goto out;
    }

    out:
    return fd;
}

/**
 * @brief Main function.
 * @return
 */
int main(void)
{
    pthread_t psdkParseThread;
    E_PsdkStat psdkStat;

    //init uart
    s_psdkUartFd = UART_Init(LINUX_UART_DEV);
    if (s_psdkUartFd == -1) {
        perror("uart init error! \r\n");
        exit(1);
    }

    FillInUserInfo(&s_testUserInfo);

    //init PSDK
    psdkStat = PsdkUpper_Init(&s_psdkUpper, &s_testUserInfo);
    if (psdkStat != PSDK_STAT_OK) {
        PSDK_LOG_ERROR("Error Psdk Init");
        exit(1);
    }

    PsdkUpper_RegSendFunction(&s_psdkUpper, sendDataToUart);
    PsdkMsgSub_Init(&s_psdkUpper);
    PsdkMsgSub_RegMsgCallbackList(&s_psdkUpper, &g_msgCallbackList);

    PsdkAppFunc_Init(&s_psdkUpper);
    PsdkAppFunc_SetReceiveFromAppCallback(&s_psdkUpper, ReceiveFromApp);

    //set fixed SKYPORT FW version
#if SKYPORT_FW_FIX_VER_NEED
    s_fixSkyport.verDebug  = SKYPORT_FW_FIX_VER_DEBUG;
    s_fixSkyport.verModify = SKYPORT_FW_FIX_VER_MODIFY;
    s_fixSkyport.verMinor  = SKYPORT_FW_FIX_VER_MINOR;
    s_fixSkyport.verMajor  = SKYPORT_FW_FIX_VER_MAJOR;

    PsdkUpper_SetFixVersion(&s_fixSkyport);
#endif

    //set PSDK product alias
#if PSDK_APP_ALIAS_NEED
    sprintf(s_productAlias.ProductAlias, PSDK_APP_ALIAS);
    PsdkUpper_SetProductAlias(&s_productAlias);
#endif

    //creat thread
    pthread_create(&psdkParseThread, NULL, psdkParse, NULL);
    pthread_join(psdkParseThread, NULL);

    return 0;
}
