/**
 ******************************************************************************
 * @file    application.c
 * @version V1.5.0
 * @date    2019/07/31
 * @brief   An example program of PSDK for demo board.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <app_config.h>
#include <psdk.h>
#include "led_button_switch.h"
#include "ms5607.h"
#include <string.h>
#include "sys.h"
#include "uart.h"
#include "log_printf.h"
#include <stdio.h>
#include <psdk_upper.h>
#include "flash.h"
#include "pps.h"
#include "app_utils.h"

#include "application.h"
#include "app_infor.h"
#include "test_app_func.h"
#include "test_msg_push.h"
#include "test_payload_cam_emu.h"
#include "test_payload_gimbal_emu.h"
#include "test_osdk_func.h"
#include "test_positioning.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

//PSDK protocol handle structure
static T_PsdkUpper s_psdkUpperHandle;

#if SKYPORT_FW_FIX_VER_NEED
//fixed SKYPORT FW version
static T_PsdkUserFixSkyport s_fixSkyport = {0};
#endif

#if PSDK_APP_ALIAS_NEED
//PSDK application alias
static T_PsdkUserCustomInfo s_productAlias;
#endif

//buffer used to receive data from UART interface
static uint8_t s_uartRecBuf[PSDK_UART_REC_BUF_LEN];
//buffer used to store data to be sent
static char s_printBuffer[512];
//PSDK user information
static T_PsdkUserInfo s_userInfo;

/* Exported variables --------------------------------------------------------*/

//a switch flag of PSDK communication channel
uint32_t g_can_uart_sw_settings = 0;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
 * @brief Check user information string and fill in user information to structure.
 * @param userInfo Pointer to user information structure to be written.
 * @return None.
 */
static void FillInUserInfo(T_PsdkUserInfo *userInfo)
{
    memset(userInfo->APP_Name, 0, sizeof(userInfo->APP_Name));
    memset(userInfo->APP_Id, 0, sizeof(userInfo->APP_Id));
    memset(userInfo->APP_Key, 0, sizeof(userInfo->APP_Key));
    memset(userInfo->DeveloperAccount, 0, sizeof(userInfo->DeveloperAccount));

    if (strlen(USER_APP_NAME) <= sizeof(userInfo->APP_Name) &&
        strlen(USER_APP_ID) <= sizeof(userInfo->APP_Id) &&
        strlen(USER_APP_KEY) <= sizeof(userInfo->APP_Key) &&
        strlen(USER_DEVELOPER_ACCOUNT) <= sizeof(userInfo->DeveloperAccount)) {
        strcpy(userInfo->APP_Name, USER_APP_NAME);
        strcpy(userInfo->APP_Id, USER_APP_ID);
        strcpy(userInfo->APP_Key, USER_APP_KEY);
        strcpy(userInfo->DeveloperAccount, USER_DEVELOPER_ACCOUNT);
    } else {
        PSDK_LOG_ERROR("Length of user information string is beyond limit. Please check.\r\n");
    }

    if (!strcmp(userInfo->APP_Name, "your_app_name") ||
        !strcmp(userInfo->APP_Id, "your_app_id") ||
        !strcmp(userInfo->APP_Key, "your_app_key") ||
        !strcmp(userInfo->DeveloperAccount, "your_developer_account")) {
        PSDK_LOG_ERROR("Please fill in correct user information to app_infor.h file.\r\n");
    }
}

/* Exported functions --------------------------------------------------------*/

/**
 * @brief Low level PSDK frame send function.
 * @param pSendData Pointer to frame to be sent.
 * @param dataLen Frame length.
 * @return PSDK function process state.
 */
E_PsdkStat Psdk_Write(const uint8_t *pSendData, uint16_t dataLen)
{
    UART_Write(PSDK_UART_NUM, pSendData, dataLen);
    return PSDK_STAT_OK;
}

/**
 * @brief Low level PSDK frame read function.
 * @param pReadData Pointer to frame to be read.
 * @param dataLen Frame length.
 * @retval -1: Read error.
 *          not -1: Counts of bytes read actually.
 */
int Psdk_Read(uint8_t *pReadData, uint16_t dataLen)
{
    int res;

    res = UART_Read(PSDK_UART_NUM, pReadData, dataLen);
    return res;
}

#if PSDK_ARCH_USE_OS

/**
 * @brief Demo board based task function.
 * @details Control the switch of PSDK communication channel and LED.
 * @param parameter
 * @return None.
 */
void RunIndicateTask(void const *parameter)
{
    uint16_t cnt = 0;
    uint32_t setting = 0;

    UNUSED(parameter);

    while (1) {
        setting = flash_read_settings();
        if (g_can_uart_sw_settings != setting) {
            flash_wirte_settings(g_can_uart_sw_settings);
            Button_StartUpdate();
        }

        OS_DELAY(1);
        if (cnt++ > 500) {
            LED_Control(LED1_GREEN, LED_OPT_TOGGLE);
            cnt = 0;
        }
    }
}

/**
 * @brief PSDK test task function.
 * @details Send sensor MS5607's data to mobile APP via transparent transmission and float window style.
 * @param parameter
 * @return None.
 */
void PsdkTestTask(void const *parameter)
{
    double temp, press;
    uint16_t realSendLen;

    UNUSED(parameter);

    Ms5607_Init();

    while (1) {
        // command channel data transmission test, get on-board temperature and pressure data, then send them to DJI Pilot
        // float window, MSDK and OSDK. Please keep packet transmission frequency less than 200Hz.
        Ms5607_GetSenorData(&temp, &press);
        snprintf(s_printBuffer, sizeof(s_printBuffer),
                 "Demo Sensor Info:\r\nBaro Pre = %f mbar\r\nBoard Temp = %f `C\r\n", press, temp);

        // push this info to DJI Pilot Floating Window
        PsdkAppFunc_PushMsgToFloatingWindow(&s_psdkUpperHandle, s_printBuffer);

        // transfer this info to MSDK
        PsdkAppFunc_TransferToApp(&s_psdkUpperHandle, (const uint8_t *) s_printBuffer,
                                  (uint16_t) (strlen(s_printBuffer) + 1), &realSendLen);

        // transfer this info to OSDK
        PsdkOsdkFunc_TransferToOsdk(&s_psdkUpperHandle, (const uint8_t *) s_printBuffer,
                                    (uint16_t) (strlen(s_printBuffer) + 1), &realSendLen);

        Test_RequestPosition(&s_psdkUpperHandle);

        OS_DELAY(1000);
    }
}

/**
 * @brief PSDK data receive task function.
 * @note All callback function process in this task.
 * @param parameter
 * @return None.
 */
void ProcessPsdkRecTask(void const *parameter)
{
    int res = 0;

    UNUSED(parameter);

    while (1) {
        res = Psdk_Read(s_uartRecBuf, sizeof(s_uartRecBuf));

        if (res > 0) {
            //===================== PSDK Recievie Function ==============================//
            //all callbacks process in this thread.
            PsdkUpper_ProcessReceiveData(&s_psdkUpperHandle, s_uartRecBuf, res);
        }

        osDelay(1);
    }
}

#else

/**
 * @brief Demo board communication interface switch function.
 * @details Control whether SKYPORT adapter connects to MCU on development board. If not, users can
 * @param None.
 * @return None.
 */
void PsdkComInfSw(void)
{
    uint32_t setting = 0;

    setting = flash_read_settings();
    if (g_can_uart_sw_settings != setting) {
        flash_wirte_settings(g_can_uart_sw_settings);
        Button_StartUpdate();
    }
}

/**
 * @brief PSDK running indicate function.
 * @param None.
 * @return None.
 */
void PsdkRunIndicate(void)
{
    LED_Control(LED1_GREEN, LED_OPT_TOGGLE);
}

/**
 * @brief PSDK test function.
 * @details Send sensor MS5607's data to mobile APP via transparent transmission and float window style.
 * @param None.
 * @return None.
 */
void PsdkTest(void)
{
    double temp, press;
    uint16_t realSendLen;

    Ms5607_GetSenorData(&temp, &press);
    snprintf(s_printBuffer, sizeof(s_printBuffer),
             "Demo Sensor Info:\r\n\r\nBaro Pre = %f mbar\r\n\r\nBoard Temp = %f `C\r\n", press, temp);

    //push this info to DJI Pilot Floating Window
    PsdkAppFunc_PushMsgToFloatingWindow(&s_psdkUpperHandle, s_printBuffer);

    //transfer this info to MSDK
    PsdkAppFunc_TransferToApp(&s_psdkUpperHandle, (const uint8_t *) s_printBuffer,
                              (uint16_t) (strlen(s_printBuffer) + 1), &realSendLen);

    // transfer this info to OSDK
    PsdkOsdkFunc_TransferToOsdk(&s_psdkUpperHandle, (const uint8_t *) s_printBuffer,
                                (uint16_t) (strlen(s_printBuffer) + 1), &realSendLen);

    Test_RequestPosition(&s_psdkUpperHandle);
}

/**
 * @brief PSDK data receive and process function.
 * @param None.
 * @return None.
 */
void PsdkProcessRec(void)
{
    int res = 0;

    res = Psdk_Read(s_uartRecBuf, sizeof(s_uartRecBuf));

    if (res > 0) {
        PsdkUpper_ProcessReceiveData(&s_psdkUpperHandle, s_uartRecBuf, res);
    }
}

#endif

/**
 * @brief Initialize the load application.
 * @param None.
 * @return None.
 */
void application_init(void)
{
    //device init
    LOG_Init();
    UART_Init(PSDK_UART_NUM, 115200);
    UART_SW_Init();
    CAN_SW_Init();

    LED_Init();
    LED_Control(LED1_GREEN, LED_OPT_OFF);
    LED_Control(LED2_RED, LED_OPT_OFF);
    LED_Control(LED3_BLUE, LED_OPT_OFF);
    Button_Init();

    g_can_uart_sw_settings = flash_read_settings();
    Button_StartUpdate();

    Ms5607Spi_Init();
    /* There have to be delay between SPI init and Ms5607 init, and os delay function is not available before starting
     * os kernel. */
#if !PSDK_ARCH_USE_OS
    MS5607_DELAY(100);
    Ms5607_Init();
#endif

    PPS_Init();

    //===================== PSDK init ==============================//
    //base init
    FillInUserInfo(&s_userInfo);
    PsdkUpper_Init(&s_psdkUpperHandle, &s_userInfo);
    PsdkUpper_RegSendFunction(&s_psdkUpperHandle, Psdk_Write);
#if !PSDK_ARCH_USE_OS
    PsdkUpper_RegReceiveFunction(&s_psdkUpperHandle, Psdk_Read);
#endif

    //app function init
    PsdkAppFunc_Init(&s_psdkUpperHandle);
    PsdkAppFunc_SetReceiveFromAppCallback(&s_psdkUpperHandle, Test_ReceiveFromMobileSDKFunc);

    PsdkAppFunc_SetWidgetList(&s_psdkUpperHandle, g_TestWidgetList, g_TestWidgetCount);
    PsdkAppFunc_RegGetWidgetValueCallback(&s_psdkUpperHandle, Test_GetWidgetValueFunc);
    PsdkAppFunc_RegSetWidgetValueCallback(&s_psdkUpperHandle, Test_SetWidgetValueFunc);
    PsdkAppFunc_RegGetTextInputBoxParamCallback(&s_psdkUpperHandle, Test_GetTextInputBoxParamFunc);

    //Msg subscribe Function init, callback functions in Application/psdk_test/test_msg_push.c
    PsdkMsgSub_Init(&s_psdkUpperHandle);
    PsdkMsgSub_RegMsgCallbackList(&s_psdkUpperHandle, &g_PushDataMsgCallbackList);

    //Camera function Emulate Test, callback functions in Application/psdk_test/test_payload_cam_emu.c
    //If want transfer video to Mobile SDK or DJI Pilot, this function must call to init Payload camera.
    PsdkCamera_Init(&s_psdkUpperHandle, &g_testCameraHandler);

    //Gimbal function Emulate Test, callback functions in Application/psdk_test/test_payload_gimbal_emu.c
    PsdkGimbal_Init(&s_psdkUpperHandle, &g_testGimbalHandler);

    //OSDK function Test, callback functions and configure parameter in Application/psdk_test/test_osdk_func.c
    PsdkOsdkFunc_Init(&s_psdkUpperHandle, &g_testOsdkFuncHandler);
    //===================== PSDK init End ============================//

    //set fixed SKYPORT FW version
#if SKYPORT_FW_FIX_VER_NEED
    s_fixSkyport.verMajor  = SKYPORT_FW_FIX_VER_MAJOR;
    s_fixSkyport.verMinor  = SKYPORT_FW_FIX_VER_MINOR;
    s_fixSkyport.verModify = SKYPORT_FW_FIX_VER_MODIFY;
    s_fixSkyport.verDebug  = SKYPORT_FW_FIX_VER_DEBUG;

    PsdkUpper_SetFixVersion(&s_fixSkyport);
#endif

    //set PSDK product alias
#if PSDK_APP_ALIAS_NEED
    strcpy(s_productAlias.ProductAlias, PSDK_APP_ALIAS);
    PsdkUpper_SetProductAlias(&s_productAlias);
#endif
}
