/**
 ******************************************************************************
 * @file    test_positioning.c
 * @version V1.5.0
 * @date    2019/07/31
 * @brief   The file simulates positioning functions for demo board.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "test_positioning.h"
#include "string.h"
#include "os_util.h"
#include "pps.h"
#include "time.h"
#include "app_utils.h"

/* Private typedef -----------------------------------------------------------*/
typedef struct {
    T_UtcTimestampMsg utcTime;
    uint32_t localTimeMs;
} T_SyncedTime;

/* Private define ------------------------------------------------------------*/
#define TIME_SYNC_EFFECTIVE_TIME_INTERVAL   500     // unit: ms
#define PAYLOAD_TARGET_POINT_COUNT          5
#define TIME_OFFSET_MAX_VALUE               2000000 // unit: us

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static T_SyncedTime syncedTime;
static bool syncedTimeExist = false;
static uint32_t eventIndex[PAYLOAD_TARGET_POINT_COUNT] = {0};

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Exported variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
 * @brief Transfer local time to UTC time based on synced time base.
 * @param syncedTimeVar: synced time base.
 * @param localTimeMs: local time to be transferred.
 * @param utcTime: Pointer to structure to store UTC time transferred.
 * @return Function process result.
 */
static E_PsdkStat Test_TransferTimeFromLocalToUtc(T_SyncedTime syncedTimeVar, uint32_t localTimeMs,
                                                  T_UtcTimestampMsg *utcTime)
{
    int32_t localTimeDiffUs = 0;
    int32_t localTimeDiffS = 0;
    int32_t tempUtcTimeUs = 0;
    struct tm syncedUtcTimeTm = {0};
    struct tm transferredUtcTimeTm = {0};
    struct tm *pTempTransferredUtcTimeTm = NULL;
    time_t syncedUtcTimeTimet = 0;
    time_t transferredUtcTimeTimet = 0;

    if (utcTime == NULL)
        return PSDK_STAT_ERR;

    localTimeDiffUs = ((int32_t) localTimeMs - (int32_t) syncedTimeVar.localTimeMs) * 1000;
    tempUtcTimeUs = (localTimeDiffUs + (int32_t) syncedTimeVar.utcTime.microsecond) % 1000000;
    utcTime->microsecond = tempUtcTimeUs < 0 ? (1000000 + tempUtcTimeUs) : tempUtcTimeUs;
    localTimeDiffS =
        ((localTimeDiffUs + (int32_t) syncedTimeVar.utcTime.microsecond) - (int32_t) utcTime->microsecond) /
        1000000;

    syncedUtcTimeTm.tm_year = syncedTimeVar.utcTime.year - 1900;
    syncedUtcTimeTm.tm_mon = syncedTimeVar.utcTime.month - 1;
    syncedUtcTimeTm.tm_mday = syncedTimeVar.utcTime.day;
    syncedUtcTimeTm.tm_hour = syncedTimeVar.utcTime.hour;
    syncedUtcTimeTm.tm_min = syncedTimeVar.utcTime.minute;
    syncedUtcTimeTm.tm_sec = syncedTimeVar.utcTime.second;
    syncedUtcTimeTm.tm_isdst = -1;
    syncedUtcTimeTimet = mktime(&syncedUtcTimeTm);
    if (syncedUtcTimeTimet == (time_t) -1)
        return PSDK_STAT_ERR;

    transferredUtcTimeTimet = syncedUtcTimeTimet + localTimeDiffS;
    pTempTransferredUtcTimeTm = localtime_r(&transferredUtcTimeTimet, &transferredUtcTimeTm);
    if (pTempTransferredUtcTimeTm == NULL)
        return PSDK_STAT_ERR;

    utcTime->year = transferredUtcTimeTm.tm_year + 1900;
    utcTime->month = transferredUtcTimeTm.tm_mon + 1;
    utcTime->day = transferredUtcTimeTm.tm_mday;
    utcTime->hour = transferredUtcTimeTm.tm_hour;
    utcTime->minute = transferredUtcTimeTm.tm_min;
    utcTime->second = transferredUtcTimeTm.tm_sec;

    LOG_TEST("Transferred Utc Time is %d.%d.%d %d:%d %d.%06d", utcTime->year, utcTime->month,
             utcTime->day, utcTime->hour, utcTime->minute, utcTime->second, utcTime->microsecond);

    return PSDK_STAT_OK;
}

/* Exported functions --------------------------------------------------------*/

/**
 * @brief Verify whether UTC timestamp package is effective.
 * @param utcTimestampComeTimeMs Time when UTC timestamp package comes.
 * @param ppsNewestTriggerTimeMs Time when the newest PPS signal be triggered.
 * @param effectiveFlag Flag specifies whether UTC timestamp package is effective.
 * @return Function process result.
 */
E_PsdkStat
Test_VerifyUtcTimestamp(uint32_t utcTimestampComeTimeMs, uint32_t ppsNewestTriggerTimeMs, uint8_t *effectiveFlag)
{
    if (effectiveFlag == NULL)
        return PSDK_STAT_ERR_PARAM;

    if (utcTimestampComeTimeMs > ppsNewestTriggerTimeMs &&
        (utcTimestampComeTimeMs - ppsNewestTriggerTimeMs) < TIME_SYNC_EFFECTIVE_TIME_INTERVAL)
        *effectiveFlag = true;
    else
        *effectiveFlag = false;

    return PSDK_STAT_OK;
}

/**
 * @brief Sync UTC time and local time when PPS signal be triggered and write to syncedTime.
 * @param utcTimestamp UTC time stamp.
 * @param ppsNewestTriggerTimeMs Time when the newest PPS signal be triggered.
 * @return Function process result.
 */
E_PsdkStat Test_SyncTime(T_UtcTimestampMsg utcTimestamp, uint32_t ppsNewestTriggerTimeMs)
{
    syncedTime.utcTime = utcTimestamp;
    syncedTime.localTimeMs = ppsNewestTriggerTimeMs;
    syncedTimeExist = true;

    return PSDK_STAT_OK;
}

/**
 * @brief Request position of target points.
 * @param psdkUpper Pointer to the top level PSDK structure.
 * @return Function process result.
 */
E_PsdkStat Test_RequestPosition(T_PsdkUpper *psdkUpper)
{
    int32_t i = 0;
    uint32_t currentLocalTime = 0;
    uint32_t timeDiffUs = 0;
    uint32_t tempTimeOfsUs = 0;
    T_UtcTimestampMsg currentUtcTime = {0};
    T_PsdkPositioningGetPosReq getPosReq = {0};
    T_PsdkPositioningGetPosAck getPosAck = {0};
    E_PsdkStat psdkStat;

    if (syncedTimeExist != true)
        return PSDK_STAT_ERR;

    if (syncedTime.localTimeMs < 2000)
        return PSDK_STAT_ERR;

    currentLocalTime = syncedTime.localTimeMs - 2000;
    Test_TransferTimeFromLocalToUtc(syncedTime, currentLocalTime, &currentUtcTime);

    getPosReq.targetPointCount = PAYLOAD_TARGET_POINT_COUNT;
    getPosReq.taskId = 0;
    getPosReq.baseTimestamp.year = currentUtcTime.year;
    getPosReq.baseTimestamp.month = currentUtcTime.month;
    getPosReq.baseTimestamp.day = currentUtcTime.day;
    getPosReq.baseTimestamp.hour = currentUtcTime.hour;
    getPosReq.baseTimestamp.minute = currentUtcTime.minute;
    getPosReq.baseTimestamp.second = currentUtcTime.second;

    do {
        tempTimeOfsUs = currentUtcTime.microsecond + timeDiffUs;
        if (tempTimeOfsUs > TIME_OFFSET_MAX_VALUE)
            continue;

        getPosReq.eventDetailInfo[i].eventId = eventIndex[i]++;
        getPosReq.eventDetailInfo[i].targetPointId = i;
        getPosReq.eventDetailInfo[i].timeOffsetUs = tempTimeOfsUs;

        LOG_TEST("Req Second is %d, Microsecond is %d.", getPosReq.baseTimestamp.second,
                 getPosReq.eventDetailInfo[i].timeOffsetUs);

        timeDiffUs += 500000;
        i++;
    } while (i < PAYLOAD_TARGET_POINT_COUNT);

    psdkStat = PsdkPositioning_GetPos(psdkUpper, &getPosReq, &getPosAck);
    if (psdkStat != PSDK_STAT_OK || getPosAck.ackCode != PSDK_CMD_ACK_CODE_OK) {
        LOG_TEST("Request Position of Target Points Fail.");
        return PSDK_STAT_ERR;
    }

    LOG_TEST("Request Position of Target Points Success.");
    LOG_TEST("Detail Position Information:");
    for (i = 0; i < PAYLOAD_TARGET_POINT_COUNT; ++i) {
        LOG_TEST("Position Solution Property: %d.", getPosAck.posDetailInfo[i].positionSolutionProperty);
        LOG_TEST("pitchAttiAngle: %d\trollAttiAngle: %d\tyawAttiAngle: %d",
                 getPosAck.posDetailInfo[i].uavAtti.pitchAttiAngle,
                 getPosAck.posDetailInfo[i].uavAtti.rollAttiAngle,
                 getPosAck.posDetailInfo[i].uavAtti.yawAttiAngle);
        LOG_TEST("northOfs: %d\tearthOfs: %d\tdownOfs: %d",
                 getPosAck.posDetailInfo[i].mainAntennaTargetPointPosOffset.northOffset,
                 getPosAck.posDetailInfo[i].mainAntennaTargetPointPosOffset.earthOffset,
                 getPosAck.posDetailInfo[i].mainAntennaTargetPointPosOffset.downOffset);
        LOG_TEST("Longitude: %f\tLatitude: %f\tHeight: %f", getPosAck.posDetailInfo[i].targetPointPos.longitude,
                 getPosAck.posDetailInfo[i].targetPointPos.latitude,
                 getPosAck.posDetailInfo[i].targetPointPos.height);
        LOG_TEST("longStdDevi: %f\tlatStdDevi: %f\thgtStdDevi: %f",
                 getPosAck.posDetailInfo[i].targetPointPosStandardDeviation.longitudeStandardDeviation,
                 getPosAck.posDetailInfo[i].targetPointPosStandardDeviation.latitudeStandardDeviation,
                 getPosAck.posDetailInfo[i].targetPointPosStandardDeviation.heightStandardDeviation);
    }

    return psdkStat;
}

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
