/**
 ******************************************************************************
 * @file    pps.c
 * @version V1.5.0
 * @date    2019/07/31
 * @brief   The file defines PPS signal related bottom layer functions.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "psdk_arch_sys.h"
#include "pps.h"
#include <stm32f4xx.h>
#include "log_printf.h"
#include "flash.h"
#include <os_util.h>

/* Private typedef -----------------------------------------------------------*/


/* Private define ------------------------------------------------------------*/
#define PPS_RCC         RCC_AHB1Periph_GPIOD
#define PPS_PORT        GPIOD
#define PPS_PIN         GPIO_Pin_2
#define PPS_EXTIPORT    EXTI_PortSourceGPIOD
#define PPS_EXTIPIN     EXTI_PinSource2
#define PPS_EXTILINE    EXTI_Line2
#define PPS_IRQn        EXTI2_IRQn
#define PPS_IRQHandler  EXTI2_IRQHandler

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static uint32_t ppsNewestTriggerTimeMs = 0;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/

/**
 * @brief PPS line initialize.
 * @param None.
 * @return None.
 */
void PPS_Init(void)
{
    EXTI_InitTypeDef EXTI_InitStructure;
    GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /* GPIO Periph clock enable */
    RCC_AHB1PeriphClockCmd(PPS_RCC, ENABLE);
    /* Enable SYSCFG clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    /* Configure PPS pin as input floating */
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;

    GPIO_InitStructure.GPIO_Pin = PPS_PIN;
    GPIO_Init(PPS_PORT, &GPIO_InitStructure);

    /* Connect EXTI pin */
    SYSCFG_EXTILineConfig(PPS_EXTIPORT, PPS_EXTIPIN);

    /* Configure EXTI Line */
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_InitStructure.EXTI_Line = PPS_EXTILINE;
    EXTI_Init(&EXTI_InitStructure);

    /* Enable and set EXTI Line2 Interrupt to the lowest priority */
    NVIC_InitStructure.NVIC_IRQChannel = PPS_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}

/**
 * @brief Get the newest PPS signal trigger time.
 * @param None.
 * @retval 0: ineffective value, specifies PPS signal have not been triggered.
 * not 0: the newest PPS signal trigger time.
 */
uint32_t PPS_GetNewestTriggerTime(void)
{
    return ppsNewestTriggerTimeMs;
}

/**
 * @brief PPS signal interrupt request handler function.
 * @param None.
 * @return None.
 */
void PPS_IRQHandler(void)
{
    if (EXTI_GetITStatus(PPS_EXTILINE) != RESET) {
        ppsNewestTriggerTimeMs = OS_GET_TIME_MS;

        /* Clear the EXTI line pending bit */
        EXTI_ClearITPendingBit(PPS_EXTILINE);
    }
}

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
